//
//
//   File: MeasureLibrary.h    Created 12-MAY-2007   Richard B. Johnson
//                                                   RichardBJohnson@comcast.net
//
//   The data, text, and source-code within these files are presumed to
//   be accurate and free of encumbrances. Use at your own risk.
//

#ifndef MEASURE_LIBRARY_H
#define MEASURE_LIBRARY_H 1

#include "IScope_Common.h"

/////////////////////////////////////////////////////////////////////////////
// METHOD NAME
//
//   AddWaveforms
//
// DESCRIPTION
//
//   This routine will mathematically add every sample of two waveforms. 
//
// INPUTS
//
//   Pointer to start of data array1
//   Pointer to start of data array2
//   Max length of array
//
// OUTPUTS
//
//   Results are stored in pdData1
/////////////////////////////////////////////////////////////////////////////
void AddWaveforms( double * pdData1, double * pdData2, long nLength );

/////////////////////////////////////////////////////////////////////////////
// METHOD NAME
//
//   SubtractWaveforms
//
// DESCRIPTION
//
//   This routine will mathematically subtract every sample of waveform
//   pdData2 from waveform pdData1. 
//
// INPUTS
//
//   Pointer to start of data array1
//   Pointer to start of data array2
//   Max length of array
//
// OUTPUTS
//
//   Results are stored in pdData1
/////////////////////////////////////////////////////////////////////////////
void SubtractWaveforms( double * pdData1, double * pdData2, long nLength );

/////////////////////////////////////////////////////////////////////////////
// METHOD NAME
//
//   MultiplyWaveforms
//
// DESCRIPTION
//
//   This routine will mathematically multiply every sample of two waveforms. 
//
// INPUTS
//
//   Pointer to start of data array1
//   Pointer to start of data array2
//   Max length of array
//
// OUTPUTS
//
//   Results are stored in pdData1
/////////////////////////////////////////////////////////////////////////////
void MultiplyWaveforms( double * pdData1, double * pdData2, long nLength );

/////////////////////////////////////////////////////////////////////////////
// METHOD NAME
//
//   InvertWaveform
//
// DESCRIPTION
//
//   This routine will mathematically multiply every sample of waveform by -1.
//
// INPUTS
//
//   Pointer to start of data array
//   Max length of array
//
// OUTPUTS
//
//   Results are stored in pdData
/////////////////////////////////////////////////////////////////////////////
void InvertWaveform( double * pdData, long nLength );

/////////////////////////////////////////////////////////////////////////////
// METHOD NAME
//
//   MeasureRiseTime
//
// DESCRIPTION
//
//   This routine will calculate the rise time of the first 
//   rising edge in the specified waveform data array.
//
// INPUTS
//
//   Pointer to data array in volts
//   Length of data array
//   Sample rate in Hz
//   LO reference voltage, if not specified the 10% voltage is used
//   HI reference voltage, if not specified the 90% voltage is used
//
// OUTPUTS
//
//   Result value, in Seconds
/////////////////////////////////////////////////////////////////////////////
double MeasureRiseTime( double * pdData, long nLength, double dRate,
                        double dLoRefVolt, 
                        double dHiRefVolt);

/////////////////////////////////////////////////////////////////////////////
// METHOD NAME
//
//   MeasureFallTime
//
// DESCRIPTION
//
//   This routine will calculate the fall time of the first 
//   falling edge in the specified waveform data array.
//
// INPUTS
//
//   Pointer to data array in volts
//   Length of data array
//   Sample rate in Hz
//   LO reference voltage, if not specified the 10% voltage is used
//   HI reference voltage, if not specified the 90% voltage is used
//
// OUTPUTS
//
//   Result value, in Seconds
/////////////////////////////////////////////////////////////////////////////
double MeasureFallTime( double * pdData, long nLength, double dRate,
                        double dLoRefVolt, 
                        double dHiRefVolt);

/////////////////////////////////////////////////////////////////////////////
// METHOD NAME
//
//   MeasureFrequency
//
// DESCRIPTION
//
//   This routine will calculate the frequency of the specified
//   waveform data array.  The frequency is determined by finding
//   the period from the first edge to the nth edge afterwards.
//   N = 1 will return the frequency of a regular repetitive waveform.
//
// INPUTS
//
//   Pointer to data array in volts
//   Length of data array
//   Sample rate in Hz
//   Number of pulses in a burst
//   Reference voltage, if not specified the 50% voltage is used
//   
//
// OUTPUTS
//
//   Result value, in Hz
/////////////////////////////////////////////////////////////////////////////
double MeasureFrequency( double * pdData, long nLength,
                         double dRate, long nBurst,
                         double dRefVolt);

/////////////////////////////////////////////////////////////////////////////
// METHOD NAME
//
//   MeasurePeriod
//
// DESCRIPTION
//
//   This routine will calculate the period from the first edge to the
//   nth edge afterwards.  N = 1 will return the period of a regular
//   repetitive waveform.
//
// INPUTS
//
//   Pointer to data array in volts
//   Length of data array
//   Sample rate in Hz
//   Number of pulses in a burst
//   Reference voltage, if not specified the 50% voltage is used
//   
//
// OUTPUTS
//
//   Result value, in Seconds
/////////////////////////////////////////////////////////////////////////////
double MeasurePeriod( double * pdData, long nLength,
                      double dRate, long nBurst,
                      double dRefVolt);

/////////////////////////////////////////////////////////////////////////////
// METHOD NAME
//
//   MeasureRMSVolt
//
// DESCRIPTION
//
//   This routine will calculate the DC voltage of a DC signal.
//   The Hi voltage will be measured of a step signal.
//   The RMS average of the specified waveform data array will be
//   calculated for all other signal types.
//
// INPUTS
//
//   Pointer to data array in volts
//   Length of data array
//   Signal type to measure
//
// OUTPUTS
//
//   Result value, in Volts
/////////////////////////////////////////////////////////////////////////////
double MeasureRMSVolt( double * pdData, long nLength,
                       enum ATE_SIGNAL_TYPE eSignalType );

/////////////////////////////////////////////////////////////////////////////
// METHOD NAME
//
//   MeasurePeakToPeak
//
// DESCRIPTION
//
//   For pulsed DC and square wave signals this will calculate the
//   delta between the Hi voltage level and the Lo voltage level.
//   All other signal types will measure the delta between the most
//   positive value to the most negative value of the specified
//   waveform data array.
//
// INPUTS
//
//   Pointer to data array in volts
//   Length of data array
//   Signal type to measure
//
// OUTPUTS
//
//   Result value, in Volts
/////////////////////////////////////////////////////////////////////////////
double MeasurePeakToPeak( double * pdData, long nLength,
                          enum ATE_SIGNAL_TYPE eSignalType );

/////////////////////////////////////////////////////////////////////////////
// METHOD NAME
//
//   MeasurePosPeak
//
// DESCRIPTION
//
//   This routine will calculate the most positive voltage 
//   of the specified waveform data array.
//
// INPUTS
//
//   Pointer to data array in volts
//   Length of data array
//
// OUTPUTS
//
//   Result value, in Volts
/////////////////////////////////////////////////////////////////////////////
double MeasurePosPeak( double * pdData, long nLength );

/////////////////////////////////////////////////////////////////////////////
// METHOD NAME
//
//   MeasureNegPeak
//
// DESCRIPTION
//
//   This routine will calculate the most negative voltage 
//   of the specified waveform data array.
//
// INPUTS
//
//   Pointer to data array in volts
//   Length of data array
//
// OUTPUTS
//
//   Result value, in Volts
/////////////////////////////////////////////////////////////////////////////
double MeasureNegPeak( double * pdData, long nLength );

/////////////////////////////////////////////////////////////////////////////
// METHOD NAME
//
//   MeasurePulseHiVoltage
//
// DESCRIPTION
//
//   This routine will calculate the pulse HI voltage of the specified 
//   waveform data array.  The signal is assumed to be digital.
//   The voltage of the middle 80% of the pulse is averaged.
//
// INPUTS
//
//   Pointer to data array in volts
//   Length of data array
//
// OUTPUTS
//
//   Result value, in Volts
/////////////////////////////////////////////////////////////////////////////
double MeasurePulseHiVoltage( double * pdData, long nLength,
                                                          enum ATE_SIGNAL_TYPE eSignalType );

/////////////////////////////////////////////////////////////////////////////
// METHOD NAME
//
//   MeasurePulseLoVoltge
//
// DESCRIPTION
//
//   This routine will calculate the pulse LO voltage of the specified 
//   waveform data array.  The signal is assumed to be digital.
//   The voltage of the middle 80% of the pulse is averaged.
//
// INPUTS
//
//   Pointer to data array in volts
//   Length of data array
//
// OUTPUTS
//
//   Result value, in Volts
/////////////////////////////////////////////////////////////////////////////
double MeasurePulseLoVoltage( double * pdData, long nLength,
                              enum ATE_SIGNAL_TYPE eSignalType );

/////////////////////////////////////////////////////////////////////////////
// METHOD NAME
//
//   MeasureDCVolt
//
// DESCRIPTION
//
//   This routine will calculate the average voltage of the specified 
//   waveform data array.
//
// INPUTS
//
//   Pointer to data array in volts
//   Length of data array
//
// OUTPUTS
//
//   Result value, in Volts
/////////////////////////////////////////////////////////////////////////////
double MeasureDCVolt( double * pdData, long nLength );

/////////////////////////////////////////////////////////////////////////////
// METHOD NAME
//
//   MeasurePulsePosWidth
//
// DESCRIPTION
//
//   This routine will calculate the positive pulse width of the 
//   first cycle in the specified waveform data array.
//
// INPUTS
//
//   Pointer to data array in volts
//   Length of data array
//   Sample rate in Hz
//   Reference voltage, if not specified the 50% voltage is used
//
// OUTPUTS
//
//   Result value, in Seconds
/////////////////////////////////////////////////////////////////////////////
double MeasurePulsePosWidth( double * pdData, long nLength, 
                             double dRate, double dRefVolt);

/////////////////////////////////////////////////////////////////////////////
// METHOD NAME
//
//   MeasurePulseNegWidth
//
// DESCRIPTION
//
//   This routine will calculate the negative pulse width of the 
//   first cycle in the specified waveform data array.
//
// INPUTS
//
//   Pointer to data array in volts
//   Length of data array
//   Sample rate in Hz
//   Reference voltage, if not specified the 50% voltage is used
//
// OUTPUTS
//
//   Result value, in Seconds
/////////////////////////////////////////////////////////////////////////////
double MeasurePulseNegWidth( double * pdData, long nLength, 
                             double dRate, double dRefVolt);

/////////////////////////////////////////////////////////////////////////////
// METHOD NAME
//
//   MeasureDutyCyclePos
//
// DESCRIPTION
//
//   This routine will calculate the duty cycle of the specified 
//   waveform data array.  Only the first cycle will be analyzed.
//
// INPUTS
//
//   Pointer to data array in volts
//   Length of data array
//   Reference voltage, if not specified the 50% voltage is used
//
// OUTPUTS
//
//   Result value, in percent
/////////////////////////////////////////////////////////////////////////////
double MeasureDutyCyclePos( double * pdData, long nLength, 
                            double dRefVolt);

/////////////////////////////////////////////////////////////////////////////
// METHOD NAME
//
//   MeasureDutyCycleNeg
//
// DESCRIPTION
//
//   This routine will calculate the duty cycle of the specified 
//   waveform data array and subtract that value from 100.0.  Only
//   the first cycle will be analyzed.
//
// INPUTS
//
//   Pointer to data array in volts
//   Length of data array
//   Reference voltage, if not specified the 50% voltage is used
//
// OUTPUTS
//
//   Result value, in percent
/////////////////////////////////////////////////////////////////////////////
double MeasureDutyCycleNeg( double * pdData, long nLength, 
                            double dRefVolt);

/////////////////////////////////////////////////////////////////////////////
// METHOD NAME
//
//   MeasurePulseAmplitude
//
// DESCRIPTION
//
//   This routine will calculate the delta from the pulse LO voltage
//   to the pulse HI voltage of the specified waveform data array.
//
// INPUTS
//
//   Pointer to data array in volts
//   Length of data array
//
// OUTPUTS
//
//   Result value, in Volts
/////////////////////////////////////////////////////////////////////////////
double MeasurePulseAmplitude( double * pdData, long nLength,
                                                          enum ATE_SIGNAL_TYPE eSignalType );

/////////////////////////////////////////////////////////////////////////////
// METHOD NAME
//
//   MeasureCycleACVolt
//
// DESCRIPTION
//
//   This routine will calculate the RMS average of the first cycle of the
//   specified waveform data array.
//
// INPUTS
//
//   Pointer to data array in volts
//   Length of data array
//   Reference voltage, if not specified the 50% voltage is used
//
// OUTPUTS
//
//   Result value, in Volts
/////////////////////////////////////////////////////////////////////////////
double MeasureCycleACVolt( double * pdData,
                           long nLength,
                           double dRefVolt);

/////////////////////////////////////////////////////////////////////////////
// METHOD NAME
//
//   MeasureCycleDCVolt
//
// DESCRIPTION
//
//   This routine will calculate the average voltage of the first cycle of
//   the specified waveform data array.
//
// INPUTS
//
//   Pointer to data array in volts
//   Length of data array
//   Reference voltage, if not specified the 50% voltage is used
//
// OUTPUTS
//
//   Result value, in Volts
/////////////////////////////////////////////////////////////////////////////
double MeasureCycleDCVolt( double * pdData,
                           long nLength,
                           double dRefVolt);

/////////////////////////////////////////////////////////////////////////////
// METHOD NAME
//
//   MeasureDCOffset
//
// DESCRIPTION
//
//   For pulsed DC signals this routine will calculate the pulse Lo
//   voltage.
//   For square wave signals this routine will calculate the average
//   of the pulse Hi voltage and the pulse Lo voltage.
//   For a step signal this routine will calculate the step Lo voltage.
//   For all other signal types this routine will calculate the average
//   of the maximum positive and maximum negative voltages.
//
// INPUTS
//
//   Pointer to data array in volts
//   Length of data array
//   Signal type to measure
//
// OUTPUTS
//
//   Result value, in Volts
/////////////////////////////////////////////////////////////////////////////
double MeasureDCOffset( double * pdData, long nLength,
                        enum ATE_SIGNAL_TYPE eSignalType );

/////////////////////////////////////////////////////////////////////////////
// METHOD NAME
//
//   MeasureOverShoot
//
// DESCRIPTION
//
//   This routine will calculate the delta from the pulse HI voltage 
//   to the peak HI voltage of the specified waveform data array.
//
// INPUTS
//
//   Pointer to data array in volts
//   Length of data array
//
// OUTPUTS
//
//   Result value, in Volts
/////////////////////////////////////////////////////////////////////////////
double MeasureOverShoot( double * pdData, long nLength, 
                                                 enum ATE_SIGNAL_TYPE eSignalType );

/////////////////////////////////////////////////////////////////////////////
// METHOD NAME
//
//   MeasureUnderShoot
//
// DESCRIPTION
//
//   This routine will calculate the delta from the pulse LO voltage 
//   to the peak LO voltage of the specified waveform data array.
//
// INPUTS
//
//   Pointer to data array in volts
//   Length of data array
//
// OUTPUTS
//
//   Result value, in Volts
/////////////////////////////////////////////////////////////////////////////
double MeasureUnderShoot( double * pdData, long nLength, 
                                                  enum ATE_SIGNAL_TYPE eSignalType );

/////////////////////////////////////////////////////////////////////////////
// METHOD NAME
//
//   MeasurePreShoot
//
// DESCRIPTION
//
//   This routine will calculate the delta from the pulse LO voltage 
//   to the peak LO voltage of the specified waveform data array.
//
// INPUTS
//
//   Pointer to data array in volts
//   Length of data array
//
// OUTPUTS
//
//   Result value, in Volts
//   This will always return 0.0
/////////////////////////////////////////////////////////////////////////////
double MeasurePreShoot( double * pdData, long nLength, 
                                                enum ATE_SIGNAL_TYPE eSignalType );

/////////////////////////////////////////////////////////////////////////////
// METHOD NAME
//
//   MeasurePhase
//
// DESCRIPTION
//
//   This routine will find the first edge of pdData1 and the corresponding
//   edge in pdData2.  The number of samples from the edge of pdData2 is
//   subtracted from the edge in pdData1.  This value is divided by the
//   number of samples between the first edge in pdData1 and the next
//   occurrence of the same slope edge in pdData1 and multiplied by 360.
//
// INPUTS
//
//   Pointer to start of data array1
//   Pointer to start of data array2
//   Max length of array
//   Reference voltage, if not specified the 50% voltage is used
//
// OUTPUTS
//
//   Result value, in Degrees
/////////////////////////////////////////////////////////////////////////////
double MeasurePhase( double * pdData1, double * pdData2, long nLength,
                     double dRefVolt);

/////////////////////////////////////////////////////////////////////////////
// METHOD NAME
//
//   MeasureTHD
//
// DESCRIPTION
//
//   This routine will calculate the power at frequency and then take the
//   ratio of the next 7 harmonics with the sum of the 7 harmonics and
//   power at frequency.
//
// INPUTS
//
//   Pointer to start of data array
//   Frequency of our primary signal
//   Sample rate
//   Max length of array
//
// OUTPUTS
//
//   Result value, in Percent
/////////////////////////////////////////////////////////////////////////////
double MeasureTHD( double * pdData, double dFreq, double dRate, long nLength );

#endif






