//
//
//   File: tester.c       Created 12-MAY-2007      Richard B. Johnson
//                                                 RichardBJohnson@comcast.net
//
//   The data, text, and source-code within these files are presumed to
//   be accurate and free of encumbrances. Use at your own risk.
//
//
//   Tests the math library used in ATEScope.
//
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <malloc.h>
#include <fcntl.h>
#include <time.h>
#ifdef linux
#include <unistd.h>
#include <sys/time.h>
#else
#include <io.h>
#include <sys/timeb.h>
extern int mkdir(const char *);
#define F_OK 0
#endif
#include "IScope_Common.h"
#include "MeasureLibrary.h"
#include "altmath.h"
#define ARRAY_SIZE 0x8000
#define LEN       (ARRAY_SIZE / 2)
#define TIMED_LEN (ARRAY_SIZE / 2)
#define NODES 0x20
#define SAMPLE_RATE 1e6
#define BIG_NUMBER 1234567890
#define TINY 1.0e-9
#define MAX_DIST 0.00099999
#if !defined(O_BINARY)
#define O_BINARY 0
#endif

static void do_files(void *buf);
static void time_fft(void);
static double get_tim(void);
static int altmath_tester(void);
static double rnd(void);

int main(int argc, char *argv[]);

static const struct {
    char *fname;
    char *kind;
    char *wind;
    int len;
    enum WINDOW_TYPE win;
    double pi;
    } fft_stuff[]={
    {"SYNF0.DAT", "Synchronous", "no",            2048, WINDOW_TYPE_NONE, PI    },
    {"SYNF1.DAT", "Synchronous", "Hanning",       2048, WINDOW_TYPE_HAN,  PI    },
    {"SYNF2.DAT", "Synchronous", "Hamming",       2048, WINDOW_TYPE_HAM,  PI    },
    {"SYNF3.DAT", "Synchronous", "raised cosine", 2048, WINDOW_TYPE_RCOS, PI    },
    {"SYNF4.DAT", "Synchronous", "Kaiser",        2048, WINDOW_TYPE_KAIS, PI    },
    {"ASYNF0.DAT","Asynchronous","Weedon/Gauss",  2048, WINDOW_TYPE_WEED, BADPI },
    {"ASYNF1.DAT","Asynchronous","Hanning",       2048, WINDOW_TYPE_HAN,  BADPI },
    {"ASYNF2.DAT","Asynchronous","Hamming",       2048, WINDOW_TYPE_HAM,  BADPI },
    {"ASYNF3.DAT","Asynchronous","raised cosine", 2048, WINDOW_TYPE_RCOS, BADPI },
    {"ASYNF4.DAT","Asynchronous","Kaiser",        2048, WINDOW_TYPE_KAIS, BADPI },
    {"ASYNF5.DAT","Asynchronous","no",            2048, WINDOW_TYPE_NONE, HIGPI },
    {"ASYNF6.DAT","Asynchronous","Hanning",       2048, WINDOW_TYPE_HAN,  HIGPI },
    {"ASYNF7.DAT","Asynchronous","Hamming",       2048, WINDOW_TYPE_HAM,  HIGPI },
    {"ASYNF8.DAT","Asynchronous","raised cosine", 2048, WINDOW_TYPE_RCOS, HIGPI },
    {"ASYNF9.DAT","Asynchronous","Kaiser",        2048, WINDOW_TYPE_KAIS, HIGPI },
    {"ASYNFA.DAT","Asynchronous","Analogic",      2048, WINDOW_TYPE_ALOG, HIGPI },
    {"ASYNFB.DAT","Asynchronous","Analogic",      2048, WINDOW_TYPE_ALOG, BADPI },
    {"SYNFC.DAT", "Synchronous", "Analogic",      2048, WINDOW_TYPE_ALOG, PI    }
     };
#define ArraySize(a) (sizeof(a) / sizeof(a[0]))

static const char Dir[]="Data/";
	 
double rrnd(void);
double rrnd()
{
    return rnd() - 0.5;
}
#ifdef linux
//
//  Return time in microseconds.
//
static double get_tim()
{
   double x;
   struct timeval tv;
   (void)gettimeofday(&tv, NULL);
   x  = ((double) tv.tv_sec) * 1e6;     // Seconds to microseconds
   x += (double) tv.tv_usec;            // Already microseconds
   return x;
}
#else
//
//  For Windoze which hasn't learned to tell time. It may return
//  time in miliseconds ((joke)), but its resolution is 1/18th of
//  a second. Therefore, it's crap for doing anything useful.
//
static double get_tim()
{
    double x;
    struct timeb tb;
    ftime(&tb);
    x =  ((double) tb.time) * 1e6;           // Seconds to microseconds 
    x += ((double) tb.millitm) * 1e3;        // Miliseconds to microseconds
    return x;
}
#endif

//
//  This times the FFT code.
//

static void time_fft()
{
    int i;
    double *win;
    double *indat;
    double *outdat;
    double theta;
    double start, total;
    double sample_rate;
    double f, dif;
    f = 0.0;
    if((win    = (double *)malloc(TIMED_LEN * sizeof(double))) == NULL)
    {
        fprintf(stderr, "Error from line %d, memory allocation\n", __LINE__);
        exit(EXIT_FAILURE);
    }
    if((indat  = (double *)malloc(TIMED_LEN * sizeof(double))) == NULL)
    {
        fprintf(stderr, "Error from line %d, memory allocation\n", __LINE__);
        exit(EXIT_FAILURE);
    }
    if((outdat = (double *)malloc(TIMED_LEN * sizeof(double))) == NULL)
    {
        fprintf(stderr, "Error from line %d, memory allocation\n", __LINE__);
        exit(EXIT_FAILURE);
    }
    MakeKaiserWIN(win, TIMED_LEN);  // Generate kernel
//
//  This is untimed data acquisition simulation (we make the data).
//
    sample_rate = (double) (TIMED_LEN / NODES);
    for(i=0; i< TIMED_LEN; i++)
    {
        theta = ((double)i / sample_rate) * 2.0 * PI;
        indat[i] = cos(theta) * 1000.0;
    }
    for(i=0; i< 4; i++)
    {
        start = get_tim();
        MultArray(indat, win, TIMED_LEN);    // Window the data
        total = get_tim() - start;
        printf("Win %d floating-point bins in %8.1f us\n",TIMED_LEN,total);
        start = get_tim();
        (void)do_FFT(outdat, indat, WINDOW_TYPE_NONE, TIMED_LEN);
        total = get_tim() - start;
        printf("FFT %d floating-point bins in %8.1f us\n", TIMED_LEN, total);
    }
    for(i=0; i< TIMED_LEN; i++)
    {
        theta = ((double)i / sample_rate) * 2.0 * PI;
        indat[i] = cos(theta) * 1000.0;
    }
    for(i=0; i< 4; i++)
    {
        start = get_tim();
        f = ArrayRMS(indat, TIMED_LEN);    // Get RMS value of data
        total = get_tim() - start;
        printf("RMS %d floating-point bins in %8.1f us\n", TIMED_LEN, total);
    }
    dif = f - (1000.0 * 1.0/sqrt(2.0));
    if((dif = (dif < 0) ? -dif:dif) > DYN)
    {
        printf("RMS value was %3.15f should have been %3.15f\n",
                f,   1000.0 * (1.0/sqrt(2.0)) );
        exit(EXIT_FAILURE);
    }
    for(i=0; i< 4; i++)
    {
        start = get_tim();
        LogN(outdat, TIMED_LEN);    // See how long this takes
        total = get_tim() - start;
        printf("LOG %d floating-point bins in %8.1f us\n", TIMED_LEN, total);
    }
    for(i=0; i< 4; i++)
    {
        start = get_tim();
        get_phase(outdat, indat, indat, TIMED_LEN);
        total = get_tim() - start;
        printf("PHA %d floating-point bins in %8.1f us\n", TIMED_LEN, total);
    }
    free(win);
    free(indat);
    free(outdat);
}

//
//  This tests the FFT and windowing code while writing output to
//  files. This lets us say that it is working correctly.
//
static void do_files(void *buf)
{
   FILE *file;
   int i, j, n, ret;
   double sample_rate, theta, max, f, *x;
   char path[0x100];
   if(access(Dir, F_OK))
	   (void)mkdir(Dir);
   for(i=0; i< ArraySize(fft_stuff); i++)
   {
        x = (double *)buf;
        sample_rate = ((double) fft_stuff[i].len / (double) NODES);
        for(j=0; j< fft_stuff[i].len; j++)
        {
            theta = ((double)j / sample_rate) * 2.0 * fft_stuff[i].pi;
            x[j] = cos(theta) * 1000.0;
        }
        f = do_FFT(x, x, fft_stuff[i].win, fft_stuff[i].len);
        ret = (int) (f + PAD);
		strcpy(path, Dir);
		strcat(path, fft_stuff[i].fname);
        if((file = fopen(path, "w")) == NULL)
        {
            fprintf(stderr, "Can't create file, %s\n", path);
            exit(EXIT_FAILURE);
        }
        n = 0;
        max = x[ret];          // Maximum value
        fprintf(file, "#@graph.title FFT/Window data\n");
        fprintf(file, "#@graph.xaxis.title Frequency\n");
        fprintf(file, "#@graph.yaxis.title Attenuation dB\n");
        fprintf(file, "#@set[%d].info %s data, %s window, length = %d\n",
                       n++, fft_stuff[i].kind, fft_stuff[i].wind, fft_stuff[i].len);
        fprintf(file, "#@set[%d].info Using %1.15f for PI.\n", n++, fft_stuff[i].pi);
        if(fft_stuff[i].win == WINDOW_TYPE_KAIS)
            fprintf(file, "#@set[%d].info Using %1.10f for Kaiser alpha.\n", n++, ALPHA);
        for(j=2; j < 5; j++)
            fprintf(file, "#@set[%d].info Harmonic number %d, at bin %-3d is %f dB\n",
                n++, j, (int)(f * (double)j), get_db(max, x[(int)((double)j * f)]));
        fprintf(file, "# Bin\tLevel (dB)\tFundamental at %f\n", f);
        LogN(x, fft_stuff[i].len);       // Log all the data
        max = x[ret];                    // Maximum value after logging
        f   = 20.0 / log(10.0);
        for(j = 0; j< fft_stuff[i].len/2; j++)
              fprintf(file, "%d\t%f\n", j,  (x[j] - max) * f );
        (void)fclose(file);
    }
}


int main(int argc, char *argv[])
{
    int i;
    double *x, *y, f;
    double sample_rate;
    double twopi;
    double theta;
    double sos;
    double ref;
    int *j, dif;
    short *sp;
    long burst;
    argv = argv;
    sample_rate = ((double) ARRAY_SIZE / (double) NODES);
    twopi = 2.0 * PI;
    sos = 1.0/ sqrt(2.0);
    f = 0.0;

    (void)altmath_tester();
    if((x = (double *) malloc(ARRAY_SIZE * sizeof(double))) == NULL)
    {
        fprintf(stderr, "Memory allocation error on line %d \n", __LINE__);
        exit(EXIT_FAILURE);
    }
    if((y = (double *) malloc(ARRAY_SIZE * sizeof(double))) == NULL)
    {
        free(x);
        fprintf(stderr, "Memory allocation error on line %d \n", __LINE__);
        exit(EXIT_FAILURE);
    }
    if((j = (int *) malloc(ARRAY_SIZE * sizeof(int))) == NULL)
    {
        free(x);
        free(y);
        fprintf(stderr, "Memory allocation error on line %d \n", __LINE__);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : CvtShort2Double()\n");
    sp = (short *) j;
    for(i=0; i< ARRAY_SIZE; i++)
        sp[i] = (short) i;
    CvtShort2Double(x, sp, ARRAY_SIZE);
    for(i=0; i< ARRAY_SIZE; i++)
    {
        if((int)x[i] != i)
        {
            fprintf(stderr, "Error at line %d\n", __LINE__);
            fprintf(stderr, "Value was %d, should have been %d\n",
                            (int) x[i], i);
            free(x);
            free(y);
            free(j);
            exit(EXIT_FAILURE);
        }
    }
    for(i=0; i< ARRAY_SIZE; i++)
        j[i] = i;
    fprintf(stdout, "Testing : CvtInt2Double()\n");
    CvtInt2Double(x, j, ARRAY_SIZE);
    for(i=0; i< ARRAY_SIZE; i++)
    {
        if((int)x[i] != i)
        {
            fprintf(stderr, "Error at line %d\n", __LINE__);
            fprintf(stderr, "Value was %d, should have been %d\n",
                            (int) x[i], i);
            free(x);
            free(y);
            free(j);
            exit(EXIT_FAILURE);
        }
    }
    fprintf(stdout, "Testing : CvtInt2Double()\n");
    CvtInt2Double(y, j, ARRAY_SIZE);
    fprintf(stdout, "Testing : AddWaveforms()\n");
    AddWaveforms(x, y, ARRAY_SIZE);
    for(i=0; i< ARRAY_SIZE; i++)
    {
        if((int)x[i] != (i * 2))
        {
            fprintf(stderr, "Error at line %d\n", __LINE__);
            fprintf(stderr, "Value was %d, should have been %d\n",
                            (int) x[i], (i * 2));
            free(x);
            free(y);
            free(j);
            exit(EXIT_FAILURE);
        }
    }
    fprintf(stdout, "Testing : SubtractWaveforms()\n");
    SubtractWaveforms(x, y, ARRAY_SIZE);
    for(i=0; i< ARRAY_SIZE; i++)
    {
        if((int)x[i] != i)
        {
            fprintf(stderr, "Error at line %d\n", __LINE__);
            fprintf(stderr, "Value was %d, should have been %d\n",
                            (int) x[i], i);
            free(x);
            free(y);
            free(j);
            exit(EXIT_FAILURE);
        }
    }
    fprintf(stdout, "Testing : CvtInt2Double()\n");
    CvtInt2Double(x, j, ARRAY_SIZE);
    fprintf(stdout, "Testing : CvtInt2Double()\n");
    CvtInt2Double(y, j, ARRAY_SIZE);
    fprintf(stdout, "Testing : MultipleWaveforms()\n");
    MultiplyWaveforms(x, y, ARRAY_SIZE);
    for(i=0; i< ARRAY_SIZE; i++)
    {
        if((int)sqrt(x[i]) != i)
        {
            fprintf(stderr, "Error at line %d\n", __LINE__);
            fprintf(stderr, "Value was %1.0f, should have been %d\n",
                             sqrt(x[i]), i);
            free(x);
            free(y);
            free(j);
            exit(EXIT_FAILURE);
        }
    }
    fprintf(stdout, "Testing : CvtInt2Double()\n");
    CvtInt2Double(x, j, ARRAY_SIZE);
    fprintf(stdout, "Testing : InvertWaveform()\n");
    InvertWaveform(x, ARRAY_SIZE);
    for(i=0; i< ARRAY_SIZE; i++)
    {
        if((int)(x[i]) != -i)
        {
            fprintf(stderr, "Error at line %d\n", __LINE__);
            fprintf(stderr, "Value was %1.0f, should have been %d\n",
                            x[i], -i);
            free(x);
            free(y);
            free(j);
            exit(EXIT_FAILURE);
        }
    }
    fprintf(stdout, "Testing : CvtInt2Double()\n");
    CvtInt2Double(x, j, ARRAY_SIZE);
    fprintf(stdout, "Testing : MeasureRiseTime()\n");
    if((int)(f = MeasureRiseTime(x, ARRAY_SIZE, 1.0, 0.0, 99.9999990)) != ARRAY_SIZE-1)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, ARRAY_SIZE-1, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasureRiseTime()\n");
    if((int)(f = MeasureRiseTime(x, ARRAY_SIZE, 1.0, 0.0, 50.0)) != ARRAY_SIZE/2)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, ARRAY_SIZE/2, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasureRiseTime()\n");
    if((int)(f = MeasureRiseTime(x, ARRAY_SIZE, 1.0, 0.0, 0.0)) != 0)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, 0, f);
        exit(EXIT_FAILURE);
    }
//
//  Yawn. Make a backward ramp.
//
    for(i=0; i< ARRAY_SIZE; i++)
        j[i] = (ARRAY_SIZE-1) - i;
    fprintf(stdout, "Testing : CvtInt2Double()\n");
    CvtInt2Double(x, j, ARRAY_SIZE);
    fprintf(stdout, "Testing : MeasureFallTime()\n");
    if((int)(f = MeasureFallTime(x, ARRAY_SIZE, 1.0, 0.0, 99.9999990)) != ARRAY_SIZE-1)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                         __LINE__, ARRAY_SIZE-1, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasureFallTime()\n");
    if((int)(f = MeasureFallTime(x, ARRAY_SIZE, 1.0, 0.0, 50.0)) != ARRAY_SIZE/2)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                         __LINE__, ARRAY_SIZE/2, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasureFallTime()\n");
    if((int)(f = MeasureFallTime(x, ARRAY_SIZE, 1.0, 0.0, 0.0)) != 1)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                         __LINE__, 1, f);
        exit(EXIT_FAILURE);
    }
    for(i=0; i< ARRAY_SIZE; i++)
        j[i] = i% (ARRAY_SIZE/NODES);
    fprintf(stdout, "Testing : CvtInt2Double()\n");
    CvtInt2Double(x, j, ARRAY_SIZE);
#define VAL (int) ((double)SAMPLE_RATE / ((double)ARRAY_SIZE/(double)NODES))
    fprintf(stdout, "Testing : MeasureFrequency()\n");
    for(i=0, burst=1; i< (NODES/2); i++, burst++)
    {
        if((int)(f = MeasureFrequency(x, ARRAY_SIZE, SAMPLE_RATE, burst, 50.0)) != VAL)
        {
            fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                              __LINE__,VAL, f);
            exit(EXIT_FAILURE);
        }
    }
    fprintf(stdout, "Testing : MeasurePeriod()\n");
    f = MeasurePeriod(x, ARRAY_SIZE, SAMPLE_RATE, NODES/2, 50.0);
    if( ((int)(1.0/f)) != VAL)
    {
        fprintf(stderr, "Error at line %d, should be %f, was %f\n",
                          __LINE__,((double)1.0/(double)VAL), f);
        exit(EXIT_FAILURE);
    }
    for(i=0; i< ARRAY_SIZE; i++)
        j[i] = (i & 1) ? BIG_NUMBER : -BIG_NUMBER;
    fprintf(stdout, "Testing : CvtInt2Double()\n");
    CvtInt2Double(x, j, ARRAY_SIZE);
    fprintf(stdout, "Testing : MeasurePeakToPeak()\n");
    if((unsigned)(f = MeasurePeakToPeak(x, ARRAY_SIZE, 0) + PAD) != (unsigned) BIG_NUMBER * 2)
    {
        fprintf(stderr,"Error at line %d, should be %u, was %1.0f\n",
                        __LINE__, (unsigned) BIG_NUMBER * 2, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : InvertWaveform()\n");
    InvertWaveform(x, ARRAY_SIZE);
    fprintf(stdout, "Testing : MeasurePeakToPeak()\n");
    if((unsigned)(f = MeasurePeakToPeak(x, ARRAY_SIZE, 0) + PAD) != (unsigned) BIG_NUMBER * 2)
    {
        fprintf(stderr,"Error at line %d, should be %u, was %1.0f\n",
                        __LINE__, (unsigned) BIG_NUMBER * 2, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasureDCVolt()\n");
    f = MeasureDCVolt(x, ARRAY_SIZE);
    if(fabs(f) > TINY)
    {
        fprintf(stderr,"Error at line %d, should be less than %e, was %e\n",
                        __LINE__, TINY, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : InvertWaveform()\n");
    InvertWaveform(x, ARRAY_SIZE);
    fprintf(stdout, "Testing : MeasureDCVolt()\n");
    f = MeasureDCVolt(x, ARRAY_SIZE);
    if(fabs(f) > TINY)
    {
        fprintf(stderr,"Error at line %d, should be less than %e, was %e\n",
                        __LINE__, TINY, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasureRMSVolt()\n");
    if((unsigned)(f = MeasureRMSVolt(x, ARRAY_SIZE, 0) + PAD) != (unsigned)BIG_NUMBER)
    {
        fprintf(stderr, "Error at line %d, should be %u, was %1.0f\n",
                        __LINE__, BIG_NUMBER, f);
        exit(EXIT_FAILURE);
    }
    for(i=0; i< ARRAY_SIZE; i++)
    {
        theta = ((double)i / sample_rate) * twopi;
        x[i] = sin(theta) * (double) BIG_NUMBER;
    }
//
//  This will show if PI and other stuff is close enough for
//  government work.
//
    fprintf(stdout, "Testing : MeasMean()\n");
    if((int)(f = MeasMean(x, ARRAY_SIZE)) != 0)
    {
        fprintf(stderr, "Error at line %d, should be %u, was %1.0f\n",
                        __LINE__, 0, f);
        exit(EXIT_FAILURE);
    }
//
//  Now see if RMS works okay, too.
//
    fprintf(stdout, "Testing : MeasureRMSVolt()\n");
    if((int)(f = MeasureRMSVolt(x, ARRAY_SIZE, 0)) != (int)(sos * (double)BIG_NUMBER))
    {
        fprintf(stderr, "Error at line %d, should be %1.0f, was %1.0f\n",
                        __LINE__, sos * (double)BIG_NUMBER, f);
        exit(EXIT_FAILURE);
    }
//
//  Now see if RMS works inverted, too.
//
    fprintf(stdout, "Testing : InvertWaveform()\n");
    InvertWaveform(x, ARRAY_SIZE);
    fprintf(stdout, "Testing : MeasureRMSVolt()\n");
    if((int)(f = MeasureRMSVolt(x, ARRAY_SIZE, 0)) != (int)(sos * (double)BIG_NUMBER))
    {
        fprintf(stderr, "Error at line %d, should be %1.0f, was %1.0f\n",
                        __LINE__, sos * (double)BIG_NUMBER, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasurePosPeak()\n");
    if((int)(f = MeasurePosPeak(x, ARRAY_SIZE)+ PAD) != (int)BIG_NUMBER)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, BIG_NUMBER, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : InvertWaveform()\n");
    InvertWaveform(x, ARRAY_SIZE);
    fprintf(stdout, "Testing : MeasurePosPeak()\n");
    if((int)(f = MeasurePosPeak(x, ARRAY_SIZE)+ PAD) != (int)BIG_NUMBER)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, BIG_NUMBER, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasureNegPeak()\n");
    if((int)(f = MeasureNegPeak(x, ARRAY_SIZE) - PAD) != (int) - BIG_NUMBER)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, BIG_NUMBER, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : InvertWaveform()\n");
    InvertWaveform(x, ARRAY_SIZE);
    fprintf(stdout, "Testing : MeasureNegPeak()\n");
    if((int)(f = MeasureNegPeak(x, ARRAY_SIZE) - PAD) != (int) - BIG_NUMBER)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, BIG_NUMBER, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasurePulseLoVoltage()\n");
    if((int)(f = MeasurePulseLoVoltage(x, ARRAY_SIZE, 0) - PAD) != (int) - BIG_NUMBER)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, BIG_NUMBER, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasurePulseHiVoltage()\n");
    if((int)(f = MeasurePulseHiVoltage(x, ARRAY_SIZE, 0) + PAD) != (int) BIG_NUMBER)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, BIG_NUMBER, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasurePulsePosWidth()\n");
    f = MeasurePulsePosWidth(x, ARRAY_SIZE, 1.0, 0.0);
    dif = (int)(f+PAD) - ((ARRAY_SIZE/NODES)/ 2);
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, (ARRAY_SIZE/NODES)/2, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasurePulsePosWidth()\n");
    f = MeasurePulsePosWidth(x, ARRAY_SIZE, 1.0, (sos * 100.0));
    dif = (int)(f+PAD) - ((ARRAY_SIZE/NODES)/ 4);
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, (ARRAY_SIZE/NODES)/4, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : InvertWaveform()\n");
    InvertWaveform(x, ARRAY_SIZE);
    fprintf(stdout, "Testing : MeasurePulsePosWidth()\n");
    f = MeasurePulsePosWidth(x, ARRAY_SIZE, 1.0, 0.0);
    dif = (int)(f+PAD) - ((ARRAY_SIZE/NODES)/ 2);
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, (ARRAY_SIZE/NODES)/2, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasurePulsePosWidth()\n");
    f = MeasurePulsePosWidth(x, ARRAY_SIZE, 1.0, (sos * 100.0));
    dif = (int)(f+PAD) - ((ARRAY_SIZE/NODES)/ 4);
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, (ARRAY_SIZE/NODES)/4, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasurePulseNegWidth()\n");
    f = MeasurePulseNegWidth(x, ARRAY_SIZE, 1.0, 0.0);
    dif = (int)(f+PAD) - ((ARRAY_SIZE/NODES)/ 2);
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, (ARRAY_SIZE/NODES)/2, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasurePulseNegWidth()\n");
    f = MeasurePulseNegWidth(x, ARRAY_SIZE, 1.0, (sos * 100.0));
    dif = (int)(f+PAD) - ((ARRAY_SIZE/NODES)/ 4);
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, (ARRAY_SIZE/NODES)/4, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : InvertWaveform()\n");
    InvertWaveform(x, ARRAY_SIZE);
    fprintf(stdout, "Testing : MeasurePulseNegWidth()\n");
    f = MeasurePulseNegWidth(x, ARRAY_SIZE, 1.0, 0.0);
    dif = (int)(f+PAD) - ((ARRAY_SIZE/NODES)/ 2);
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, (ARRAY_SIZE/NODES)/2, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasurePulseNegWidth()\n");
    f = MeasurePulseNegWidth(x, ARRAY_SIZE, 1.0, (sos * 100.0));
    dif = (int)(f+PAD) - ((ARRAY_SIZE/NODES)/ 4);
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, (ARRAY_SIZE/NODES)/4, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasureDutyCyclePos()\n");
    f = MeasureDutyCyclePos(x, ARRAY_SIZE, 50.0);
    dif = (int)(f+PAD) - 50;
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, 50, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasureDutyCyclePos()\n");
    f = MeasureDutyCyclePos(x, ARRAY_SIZE, (sos * 100.0));
    dif = (int)(f+PAD) - 36;
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, 36, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasureDutyCyclePos()\n");
    f = MeasureDutyCyclePos(x, ARRAY_SIZE, 100.0);
    dif = (int)f;
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, 0, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : InvertWaveform()\n");
    InvertWaveform(x, ARRAY_SIZE);
    fprintf(stdout, "Testing : MeasureDutyCyclePos()\n");
    f = MeasureDutyCyclePos(x, ARRAY_SIZE, 50.0);
    dif = (int)(f+PAD) - 50;
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, 50, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasureDutyCyclePos()\n");
    f = MeasureDutyCyclePos(x, ARRAY_SIZE, (sos * 100.0));
    dif = (int)(f+PAD) - 36;
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, 36, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasureDutyCyclePos()\n");
    f = MeasureDutyCyclePos(x, ARRAY_SIZE, 100.0);
    dif = (int)f;
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, 0, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasureDutyCycleNeg()\n");
    f = MeasureDutyCycleNeg(x, ARRAY_SIZE, 50.0);
    dif = (int)(f+PAD) - 50;
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, 50, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasureDutyCycleNeg()\n");
    f = MeasureDutyCycleNeg(x, ARRAY_SIZE, sos * 100.0);
    dif = (int)(f+PAD) - 64;
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, 64, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasureDutyCycleNeg()\n");
    f = MeasureDutyCycleNeg(x, ARRAY_SIZE, 100.0);
    dif = (int)(f - 100.0);
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, 100, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : InvertWaveform()\n");
    InvertWaveform(x, ARRAY_SIZE);
    fprintf(stdout, "Testing : MeasureDutyCycleNeg()\n");
    f = MeasureDutyCycleNeg(x, ARRAY_SIZE, 50.0);
    dif = (int)(f+PAD) - 50;
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, 50, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasureDutyCycleNeg()\n");
    f = MeasureDutyCycleNeg(x, ARRAY_SIZE, sos * 100.0);
    dif = (int)(f+PAD) - 64;
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, 64, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasureDutyCycleNeg()\n");
    f = MeasureDutyCycleNeg(x, ARRAY_SIZE, 100.0);
    dif = (int)(f - 100);
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, 100, f);
        exit(EXIT_FAILURE);
    }
    for(i=0; i< ARRAY_SIZE; i++)
        j[i] = i% ((ARRAY_SIZE/NODES + 1));
    fprintf(stdout, "Testing : CvtInt2Double()\n");
    CvtInt2Double(y, j, ARRAY_SIZE);
    fprintf(stdout, "Testing : MeasurePulseAmplitude()\n");
    f = MeasurePulseAmplitude(y, ARRAY_SIZE,0 );
    dif = (int)(f + PAD) - (ARRAY_SIZE/NODES);
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, ARRAY_SIZE/NODES, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasureCycleACVolt()\n");
    f = MeasureCycleACVolt(x, ARRAY_SIZE, 0.0);
    dif = (int) ((f + PAD) - (sos * (double)BIG_NUMBER));
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %1.0f, was %1.0f\n",
                        __LINE__, (sos * (double)BIG_NUMBER), f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasureCycleACVolt()\n");
    f = MeasureCycleACVolt(x, ARRAY_SIZE, 50.0);
    dif = (int) ((f + PAD) - (sos * (double)BIG_NUMBER));
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %1.0f, was %1.0f\n",
                        __LINE__, (sos * (double)BIG_NUMBER), f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasureCycleACVolt()\n");
    f = MeasureCycleACVolt(x, ARRAY_SIZE, 99.0);
    dif = (int) ((f + PAD) - (sos * (double)BIG_NUMBER));
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %1.0f, was %1.0f\n",
                        __LINE__, (sos * (double)BIG_NUMBER), f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : InvertWaveform()\n");
    InvertWaveform(x, ARRAY_SIZE);
    fprintf(stdout, "Testing : MeasureCycleACVolt()\n");
    f = MeasureCycleACVolt(x, ARRAY_SIZE, 0.0);
    dif = (int) ((f + PAD) - (sos * (double)BIG_NUMBER));
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %1.0f, was %1.0f\n",
                        __LINE__, (sos * (double)BIG_NUMBER), f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasureCycleACVolt()\n");
    f = MeasureCycleACVolt(x, ARRAY_SIZE, 50.0);
    dif = (int) ((f + PAD) - (sos * (double)BIG_NUMBER));
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %1.0f, was %1.0f\n",
                        __LINE__, (sos * (double)BIG_NUMBER), f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasureCycleACVolt()\n");
    f = MeasureCycleACVolt(x, ARRAY_SIZE, 99.0);
    dif = (int) ((f + PAD) - (sos * (double)BIG_NUMBER));
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %1.0f, was %1.0f\n",
                        __LINE__, (sos * (double)BIG_NUMBER), f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasureCycleDCVolt()\n");
    f = MeasureCycleDCVolt(x, ARRAY_SIZE, 50.0);
    dif = (int) f;
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, 0, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasureCycleDCVolt()\n");
    f = MeasureCycleDCVolt(x, ARRAY_SIZE, 50.0);
    dif = (int) f;
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, 0, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasureDCOffset()\n");
    f = MeasureDCOffset(x, ARRAY_SIZE, SIGNAL_AC);
    dif = (int) f;
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, 0, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasureDCOffset()\n");
    f = MeasureDCOffset(x, ARRAY_SIZE, SIGNAL_DC);
    dif = (int) (f + (double) BIG_NUMBER);
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, -BIG_NUMBER, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasureDCOffset()\n");
    f = MeasureDCOffset(x, ARRAY_SIZE, SIGNAL_SQUARE);
    dif = (int) f;
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %1.0f\n",
                        __LINE__, 0, f);
        exit(EXIT_FAILURE);
    }
    for(i=0; i< ARRAY_SIZE; i++)
    {
        theta = ((double)i / sample_rate) * twopi;
        y[i] = cos(theta) * (double) BIG_NUMBER;
    }
    InvertWaveform(y, ARRAY_SIZE);
    fprintf(stdout, "Testing : MeasurePhase()\n");
    ref = 0.0;
    f = MeasurePhase(x, y, ARRAY_SIZE, ref);
    dif = (int) (f - 90.0);
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %f, was %f\n",
                        __LINE__, 90.0, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasurePhase()\n");
    f = MeasurePhase(y, x, ARRAY_SIZE, ref);
    if(f < 0) f += 360.0;
    dif = (int) (f - 270.0);
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %f, was %f\n",
                        __LINE__, 270.0, f);
        exit(EXIT_FAILURE);
    }
    memcpy(y, x, ARRAY_SIZE * sizeof(double));
    fprintf(stdout, "Testing : InvertWaveform()\n");
    InvertWaveform(y, ARRAY_SIZE);
    fprintf(stdout, "Testing : MeasurePhase()\n");
    f = MeasurePhase(x, y, ARRAY_SIZE, ref);
    dif = (int) (f - 180.0);
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %f, was %f\n",
                        __LINE__, 180.0, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasurePhase()\n");
    f = MeasurePhase(x, x, ARRAY_SIZE, ref);
    dif = (int) (f);
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %f, was %f\n",
                        __LINE__, 0.0, f);
        exit(EXIT_FAILURE);
    }
    for(i=0; i< 4; i++)
    {
        fprintf(stdout, "Testing : MeasurePhase(%f)\n", ref);
        f = MeasurePhase(x, x, ARRAY_SIZE, ref);
        dif = (int) (f);
        if((dif = (dif < 0) ? -dif:dif) > 1)
        {
            fprintf(stderr, "Error at line %d, should be %f, was %f\n",
                            __LINE__, 0.0, f);
            exit(EXIT_FAILURE);
        }
        fprintf(stdout, "Testing : MeasurePhase(%f)\n", ref);
        f = MeasurePhase(x, y, ARRAY_SIZE, ref);
        dif = (int) (f - 180.0);
        if((dif = (dif < 0) ? -dif:dif) > 1)
        {
            fprintf(stderr, "Error at line %d, should be %f, was %f\n",
                            __LINE__, 180.0, f);
            exit(EXIT_FAILURE);
        }
        ref += 30.0;
    }
    for(i=0; i< ARRAY_SIZE; i++)
    {
        theta = ((double)i / sample_rate) * twopi;
        y[i] = cos(theta) * (double) BIG_NUMBER;
    }
    InvertWaveform(y, ARRAY_SIZE);
    ref = 10.0;
    for(i=0; i< 4; i++)
    {
        fprintf(stdout, "Testing : MeasurePhase(%f)\n", ref);
        f = MeasurePhase(x, y, ARRAY_SIZE, ref);
        dif = (int) (f - 90.0);
        if((dif = (dif < 0) ? -dif:dif) > 1)
        {
            fprintf(stderr, "Error at line %d, should be %f, was %f\n",
                            __LINE__, 90.0, f);
            exit(EXIT_FAILURE);
        }
        ref += 30.0;
    }
//
//  Make array of square waves.
//
    for(i=0; i< ARRAY_SIZE; i++)
        j[i] = (i %100  < 50) ? 100 : -100;
    j[201] += 20;     // Add overshoot to one element
    j[202] += 20;     // Add overshoot to next element
    fprintf(stdout, "Testing : CvtInt2Double()\n");
    CvtInt2Double(x, j, ARRAY_SIZE);
    for(i=0; i< ARRAY_SIZE; i++)
        x[i] += rrnd() * 1.0;                // Add 1 percent noise 
    AddConst(x, -10.0, ARRAY_SIZE);          // Add 10 percent offset
    fprintf(stdout, "Testing : MeasureOverShoot()\n");
    f = MeasureOverShoot(x, ARRAY_SIZE, 0);
    printf("%2.2f  \r", f);
    fflush(stdout);
    dif = (int) (f + PAD - 10.0);
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %f, was %f\n",
                         __LINE__, 10.0, f);
        exit(EXIT_FAILURE);
    }
//
//  Make array of narrow glitches.
//
    for(i=0; i< ARRAY_SIZE; i++)
        j[i] = (i & 1) ? 10000 : -10000;
    j[201] += 100;     // Add overshoot to one element
    fprintf(stdout, "Testing : CvtInt2Double()\n");
    CvtInt2Double(x, j, ARRAY_SIZE);
    f = MeasureOverShoot(x, ARRAY_SIZE, 0);
    dif = (int) (f + PAD - 0.50);
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %f, was %f\n",
                         __LINE__, 0.50, f);
        exit(EXIT_FAILURE);
    }
    for(i=0; i< ARRAY_SIZE; i++)
        j[i] = (i %100  < 50) ? 10000 : -10000;
    j[201] += 2000;     // Add overshoot to one element
    fprintf(stdout, "Testing : CvtInt2Double()\n");
    CvtInt2Double(x, j, ARRAY_SIZE);
    InvertWaveform(x, ARRAY_SIZE);
    fprintf(stdout, "Testing : MeasureUnderShoot()\n");
    f = MeasureUnderShoot(x, ARRAY_SIZE, 0);
    dif = (int) (f + PAD - 10.0);
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %f, was %f\n",
                         __LINE__, 1.0, f);
        exit(EXIT_FAILURE);
    }
    for(i=0; i< ARRAY_SIZE; i++)
    {
        theta = ((double)i / sample_rate) * twopi;
        x[i] = sin(theta) * 10.0;
    }
    fprintf(stdout, "Testing : do_FFT(synchronous)\n");
    i = (int) do_FFT(y, x, WINDOW_TYPE_HAN, ARRAY_SIZE);
    dif = i - NODES;
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %d\n",
                         __LINE__, NODES, i);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasureTHD(synchronous)\n");
    f = MeasureTHD(x, (double)NODES, sample_rate, ARRAY_SIZE);
    if(f > MAX_DIST)
    {
        fprintf(stderr, "Error at line %d, should be less than %f, was %f\n",
                         __LINE__, MAX_DIST, f);
        exit(EXIT_FAILURE);
    }
    for(i=0; i< ARRAY_SIZE; i++)
    {
        theta = ((double)i * 2.0 / sample_rate) * twopi;
        x[i] += sin(theta) * 3.33333;
    }
    fprintf(stdout, "Testing : MeasureTHD(synchronous)\n");
    f = MeasureTHD(x, (double)NODES, sample_rate, ARRAY_SIZE);
    ref = f - 33.33333;
    ref = (ref < 0.0) ? -ref:ref;
    if(ref > 1.0)
    {
        fprintf(stderr, "Error at line %d, should be %f, was %f\n",
                         __LINE__, 33.333, f);
        exit(EXIT_FAILURE);
    }
    for(i=0; i< ARRAY_SIZE; i++)
    {
        theta = ((double)i / sample_rate) * 2.0 * BADPI;
        x[i] = sin(theta) * 10.0;
    }
    fprintf(stdout, "Testing : do_FFT(asynchrnous)\n");
    i = (int) do_FFT(y, x, WINDOW_TYPE_HAM, ARRAY_SIZE);
    dif = i - NODES;
    if((dif = (dif < 0) ? -dif:dif) > 1)
    {
        fprintf(stderr, "Error at line %d, should be %d, was %d\n",
                         __LINE__, NODES, i);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : MeasureTHD(asynchronous)\n");
    f = MeasureTHD(x, (double)NODES, sample_rate, ARRAY_SIZE);
    if(f > MAX_DIST)
    {
        fprintf(stderr, "Error at line %d, should be less than %f, was %f\n",
                         __LINE__, MAX_DIST, f);
        exit(EXIT_FAILURE);
    }
    for(i=0; i< ARRAY_SIZE; i++)
    {
        theta = ((double)i * 2.0 / sample_rate) * 2.0 * BADPI;
        x[i] += sin(theta) * 3.33333;
    }
    fprintf(stdout, "Testing : MeasureTHD(asynchrnous)\n");
    f = MeasureTHD(x, (double)NODES, sample_rate, ARRAY_SIZE);
    ref = f - 33.33333;
    ref = (ref < 0.0) ? -ref:ref;
    if(ref > 1.0)
    {
        fprintf(stderr, "Error at line %d, should be %f, was %f\n",
                         __LINE__, 33.333, f);
        exit(EXIT_FAILURE);
    }
    for(i=0; i< ARRAY_SIZE; i++)
    {
        theta = ((double)i / sample_rate) * twopi;
        x[i] = sin(theta) * 10.0;
    }
    fprintf(stdout, "Testing : MeasureTHD(synchronous)\n");
    f = MeasureTHD(x, (double)NODES, sample_rate, ARRAY_SIZE);
    if(f > MAX_DIST)
    {
        fprintf(stderr, "Error at line %d, should be less than %f, was %f\n",
                         __LINE__, MAX_DIST, f);
        exit(EXIT_FAILURE);
    }
    for(i=0; i< ARRAY_SIZE; i++)
    {
        theta = ((double)i * 6.0 / sample_rate) * twopi;
        x[i] += sin(theta) * 3.33333;
    }
    fprintf(stdout, "Testing : MeasureTHD(synchronous)\n");
    f = MeasureTHD(x, (double)NODES, sample_rate, ARRAY_SIZE);
    ref = f - 33.33333;
    ref = (ref < 0.0) ? -ref:ref;
    if(ref > 1.0)
    {
        fprintf(stderr, "Error at line %d, should be %f, was %f\n",
                         __LINE__, 33.333, f);
        exit(EXIT_FAILURE);
    }
    for(i=0; i< ARRAY_SIZE; i++)
    {
        theta = ((double)i / sample_rate) * 2.0 * BADPI;
        x[i] = sin(theta) * 10.0;
    }
    fprintf(stdout, "Testing : MeasureTHD(asynchronous)\n");
    f = MeasureTHD(x, (double)NODES, sample_rate, ARRAY_SIZE);
    if(f > MAX_DIST)
    {
        fprintf(stderr, "Error at line %d, should be less than %f, was %f\n",
                         __LINE__, MAX_DIST, f);
        exit(EXIT_FAILURE);
    }
    for(i=0; i< ARRAY_SIZE; i++)
    {
        theta = ((double)i * 8.0 / sample_rate) * 2.0 * BADPI;
        x[i] += sin(theta) * 3.33333;
    }
    fprintf(stdout, "Testing : MeasureTHD(asynchronous)\n");
    f = MeasureTHD(x, (double)NODES, sample_rate, ARRAY_SIZE);
    ref = f - 33.33333;
    ref = (ref < 0.0) ? -ref:ref;
    if(ref > 1.0)
    {
        fprintf(stderr, "Error at line %d, should be %f, was %f\n",
                         __LINE__, 33.333, f);
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "Testing : Writing FFT files\n");
    do_files(x);
    fprintf(stdout, "Testing : Timing FFT routines\n");
    time_fft();
    free(x);
    free(y);
    free(j);
    fprintf(stdout, "Everything tested okay!\n");
    return 0;
}

//
//   Returns a random-number between about 0.0 and 1.0
//
static double rnd()
{
    double x;
    x = (double)rand() / (double)RAND_MAX;
    return x;
}


static struct {
	const char *fname;
	void (*funct)(double *, int);
} window[] = {
	{"RCOSKRNL.DAT", MakeRaisedCOS      },    
	{"WEEDKRN.DAT",  MakeWeedonGauss    },
	{"ANALOGIC.DAT", MakeAnalogicWIN    },
	{"KAISER.DAT",   MakeKaiserWIN      },
	{"HAMMING.DAT",  MakeHammingWIN     },
	{"HANNING.DAT",  MakeHanningWIN     },
	{"BLACKHRS.DAT", MakeBlackmanHarris },
	{"BLACKMAN.DAT", MakeBlackman       },
	{"NUTALL.DAT",   MakeNutall         },
	{"BLACKNUT.DAT", MakeBlackNutall    },
	{"LANCZOS.DAT",  MakeLanczos        },
	{"RECTANG.DAT",  MakeRectWIN        },
};

static int altmath_tester()
{
    double *x, y;
    short *sp;
    int   *ip;
    size_t i, j;
    FILE *file;
	char path[0x100];

    if((x = (double *) malloc(LEN * sizeof(double))) == NULL)
    {
        fprintf(stderr, "Memory allocation error\n");
        exit(EXIT_FAILURE);
    }
    if((sp = (short *) malloc(LEN * sizeof(short))) == NULL)
    {
        fprintf(stderr, "Memory allocation error\n");
        exit(EXIT_FAILURE);
    }
    if((ip = (int *) malloc(LEN * sizeof(int))) == NULL)
    {
        fprintf(stderr, "Memory allocation error\n");
        exit(EXIT_FAILURE);
    }
    (void)srand(time(NULL));
    printf("Filling array\n");
    for(i=0; i< LEN; i++)
        x[i] =  rnd();
    printf("Random numbers, %d to %d\n", 0, 1);
    printf("        Mean = %f\n", MeasMean(x, LEN));
    printf("    Variance = %f\n", MeasVariance(x, LEN));
    printf("Standard dev = %f\n", MeasStdDev(x, LEN));
    y = (double) (-LEN/2) + 0.5;
    printf("Filling array\n");
    for(i=0; i< LEN; i++)
    {
        x[i] =  y + (double)i;
    }
    printf("Linear numbers, %f to %f\n", y, y + (double)LEN-1);
    printf("        Mean = %f\n", MeasMean(x, LEN));
    printf("    Variance = %f\n", MeasVariance(x, LEN));
    printf("Standard dev = %f\n", MeasStdDev(x, LEN));
    printf("Filling array\n");
    for(i=0; i< LEN; i++)
    {
        if(i & 1)
            x[i] = -1.0;
        else
            x[i] = 1.0;
    }
    printf("+1.0 and -1.0\n");
    printf("        Mean = %f\n", MeasMean(x, LEN));
    printf("    Variance = %f\n", MeasVariance(x, LEN));
    printf("Standard dev = %f\n", MeasStdDev(x, LEN));
    memset(x, 0x00, LEN * sizeof(double));
    printf("Test time to a positive-going burp\n");
    x[LEN/2] = 1.0;
    printf("Time to positive burp = %d\n", TimeToReference(x, 0.5, LEN));
    printf("Time to positive burp = %d\n", TimeToReference(x, -0.5, LEN));
    printf("Test time to a negative-going burp\n");
    x[LEN/2] = -1.0;
    printf("Time to negative burp = %d\n", TimeToReference(x, -0.5, LEN));
    printf("Time to negative burp = %d\n", TimeToReference(x, 0.5, LEN));

    x[LEN/2] = -1.0;
    x[LEN/3] = -1.0;
    x[LEN/4] = -1.0;
    x[LEN/5] = -1.0;
    x[LEN/6] = -1.0;
    x[LEN/7] = -1.0;
    x[LEN/8] = -1.0;
    x[LEN/9] = -1.0;     /* 8 bumps, 16 crossings */
    printf("Number of crossings = %d\n", NrCrossRefLevel(x, -0.5, LEN));
    printf("Number of crossings = %d\n", NrCrossRefLevel(x,  0.5, LEN));
    printf("Number of crossings = %d\n", NrCrossRefLevel(x,  -0.0000000001, LEN));
    for(i=0; i< LEN; i++)
        x[i] =  rnd();
    printf("Number of bumps = %d\n", NrCrossRefLevel(x, 0.1, LEN ));
    printf("Number of bumps = %d\n", NrCrossRefLevel(x, 0.2, LEN ));
    printf("Number of bumps = %d\n", NrCrossRefLevel(x, 0.3, LEN ));
    printf("Number of bumps = %d\n", NrCrossRefLevel(x, 0.4, LEN ));
    printf("Number of bumps = %d\n", NrCrossRefLevel(x, 0.5, LEN ));
    printf("Number of bumps = %d\n", NrCrossRefLevel(x, 0.6, LEN ));
    printf("Number of bumps = %d\n", NrCrossRefLevel(x, 0.7, LEN ));
    memset(x, 0x00, LEN * sizeof(double));
    printf("Time to positive burp = %d\n", TimeToReference(x, 0.5, LEN));
    for(i=0; i < 10; i++)
    {
        x[i] = 1.0;
        printf("Time to positive burp = %d\n", TimeToReference(x, 0.1, LEN));
        x[i] = 0.0;
    }
    for(i=0; i < 10; i++)
    {
        x[i] = 1.0;
        printf("Time to Pos Ref = %d\n", TimeToPosRef(x, 0.1, LEN));
        x[i] = 0.0;
    }
    for(i=0; i < 10; i++)
    {
        x[i] = -1.0;
        printf("Time to Neg Ref = %d\n", TimeToNegRef(x, -0.1, LEN));
        x[i] = 0.0;
    }
    for(i=0; i< LEN; i++)
        x[i] =  rnd();

    printf("Max slope = %d\n",TimeMaxSlope(x, LEN));
    memset(x, 0x00, LEN * sizeof(double));
    x[3] = 1.0;
    printf("Max slope = %d\n",TimeMaxSlope(x, LEN));
    x[3] = 0.0;
    for(i=0; i < 20; i++)
    {
        x[i] = 123.456;
        printf ("Limits = %d\n", WithinLimits(x, 123.457, 123.455, LEN));
        x[i] = 0.0;
    }
    x[0] = 1.0;
    x[100] = 1.1;
    x[101] = 1.0;
    x[102] = 1.0;
    printf ("Settle = %d\n", SettlingTime(x, 0.001, LEN));
    for(i=0; i< LEN; i++)
        x[i] =  rnd();
    printf ("Settle = %d\n", SettlingTime(x, 0.01, LEN));
    printf ("Settle = %d\n", SettlingTime(x, 0.001, LEN));
    printf ("Settle = %d\n", SettlingTime(x, 0.0001, LEN));
    printf ("Settle = %d\n", SettlingTime(x, 0.00001, LEN));
    printf ("Settle = %d\n", SettlingTime(x, 0.000001, LEN));
    printf ("Settle = %d\n", SettlingTime(x, 0.0000001, LEN));
    printf ("Settle = %d\n", SettlingTime(x, 0.00000001, LEN));

    for(i=0; i< LEN; i++)
    {
        ip[i] = (int) i;
        sp[i] = (short) i;
    }
    printf("Testing double conversion of %d integer elements\n", LEN);
    CvtInt2Double(x, ip, LEN);
    for(i=0; i< LEN; i++)
        if(ip[i] != (int) x[i])
        {
            printf("Compare failed at %d[%d] != %f[%d]\n", ip[i], i, x[i], i);
            free(sp);
            free(ip);
            free(x);
            exit(EXIT_FAILURE);
        }
    printf("Testing double conversion of %d short integer elements\n", LEN);
    CvtShort2Double(x, sp, LEN);
    for(i=0; i< LEN; i++)
        if(sp[i] != (short) x[i])
        {
            printf("Compare failed at %d[%d] != %f[%d]\n",(int)sp[i], i, x[i], i);
            free(sp);
            free(ip);
            free(x);
            exit(EXIT_FAILURE);
        }
    CvtInt2Double(x, ip, LEN);
    printf("Testing adding a constant to %d elements\n", LEN);
    AddConst(x, 99.0, LEN);
    for(i=0; i< LEN; i++)
        if((ip[i] + 99 ) != (int) x[i])
        {
            printf("Compare failed at %d[%d] != %f[%d]\n", ip[i], i, x[i], i);
            free(sp);
            free(ip);
            free(x);
            exit(EXIT_FAILURE);
        }
    CvtInt2Double(x, ip, LEN);
    printf("Testing subtracting a constant from %d elements\n", LEN);
    SubConst(x, 99.0, LEN);
    for(i=0; i< LEN; i++)
        if((ip[i] - 99 ) != (int) x[i])
        {
            printf("Compare failed at %d[%d] != %f[%d]\n", ip[i], i, x[i], i);
            free(sp);
            free(ip);
            free(x);
            exit(EXIT_FAILURE);
        }
    CvtInt2Double(x, ip, LEN);
    printf("Testing multiply by a constant for %d elements\n", LEN);
    MulConst(x, 99.0, LEN);
    for(i=0; i< LEN; i++)
        if((ip[i] * 99 ) != (int) x[i])
        {
            printf("Compare failed at %d[%d] != %f[%d]\n", ip[i], i, x[i], i);
            free(sp);
            free(ip);
            free(x);
            exit(EXIT_FAILURE);
        }
    CvtInt2Double(x, ip, LEN);
    printf("Testing dividing by a constant for %d elements\n", LEN);
    DivConst(x, 99.0, LEN);
    for(i=0; i< LEN; i++)
        if((ip[i] / 99 ) != (int) x[i])
        {
            printf("Compare failed at %d[%d] != %f[%d]\n", ip[i], i, x[i], i);
            free(sp);
            free(ip);
            free(x);
            exit(EXIT_FAILURE);
        }
    printf("Testing multiplication of %d elements\n", LEN);
    CvtInt2Double(x, ip, LEN);
    MultArray(x, x, LEN);
    for(i=0; i< LEN; i++)
        if((ip[i] * ip[i] ) != (int) x[i])
        {
            printf("Compare failed at %d[%d] != %f[%d]\n", ip[i], i, x[i], i);
            free(sp);
            free(ip);
            free(x);
            exit(EXIT_FAILURE);
        }
	for(i=0; i< ArraySize(window); i++)
	{
    	strcpy(path, Dir);
	    strcat(path, window[i].fname);
		(void)unlink(path);
        if((file = fopen(path, "w")) != NULL)
		{
            (*window[i].funct)(x, LEN);
            for(j=0; j< LEN; j++)
                fprintf(file, "%d\t%2.8f\n", j, x[j]);
            fclose(file);
		}
	}
    for(i=0; i< LEN; i++)
        x[i] = 10.0;
    printf("Testing logging of %d elements\n", LEN);
    Log10(x, LEN);
    for(i=0; i< LEN; i++)
        if(x[i] != 1.0)
        {
            printf("Compare failed at %d %f\n", i,  x[i]);
            printf("Was %f, should have been %f\n", x[i],  1.0);
            exit(EXIT_FAILURE);
        }
    for(i=0; i< LEN; i++)
        x[i] = 10.0;
    printf("Testing logging of %d elements\n", LEN);
    LogN(x, LEN);
    y = log(10.0);
    for(i=0; i< LEN; i++)
        if(fabs (x[i] - y) > DYN)
        {
            printf("Compare failed at %d %f\n", i,  x[i]);
            printf("Was %f, should have been %f\n", x[i],  y);
            exit(EXIT_FAILURE);
        }
    free(sp);
    free(ip);
    free(x);
    return 0;
}


